@@ -7,7 +7,7 @@
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -37,6 +37,8 @@
 
 #include <corosync/engine/objdb.h>
 #include <corosync/engine/logsys.h>
+#include <corosync/list.h>
+#include <corosync/engine/coroapi.h>
 
 /*
  * All service handlers in the AIS
@@ -49,30 +51,24 @@ struct dynamic_service {
 };
 #define MAX_DYNAMIC_SERVICES 128
 
-struct main_config {
-	/*
-	 * logging configuration
-	 */
-	char *logfile;
-	unsigned int logmode;
-	unsigned int syslog_facility;
-	unsigned int minimum_priority;
-
-	/*
-	 * user/group to run as
-	 */
+/*
+ * Structure describing cached uidgid item
+ */
+struct uidgid_item {
+	struct list_head list;
 	int uid;
 	int gid;
 };
 
-int corosync_main_config_read_logging (
-	struct objdb_iface_ver0 *objdb,
-	const char **error_string,
-	struct main_config *main_config);
+extern struct list_head uidgid_list_head;
 
 extern int corosync_main_config_read (
 	struct objdb_iface_ver0 *objdb,
-	const char **error_string,
-	struct main_config *main_config);
+	const char **error_string);
+
+extern int corosync_main_config_compatibility_read (
+	struct objdb_iface_ver0 *objdb,
+	enum cs_sync_mode *minimum_sync_mode,
+	const char **error_string);
 
 #endif /* MAINCONFIG_H_DEFINED */
