@@ -41,6 +41,19 @@
 #include <stdio.h>
 #include <corosync/hdb.h>
 
+typedef enum {
+	OBJDB_VALUETYPE_INT16,
+	OBJDB_VALUETYPE_UINT16,
+	OBJDB_VALUETYPE_INT32,
+	OBJDB_VALUETYPE_UINT32,
+	OBJDB_VALUETYPE_INT64,
+	OBJDB_VALUETYPE_UINT64,
+	OBJDB_VALUETYPE_FLOAT,
+	OBJDB_VALUETYPE_DOUBLE,
+	OBJDB_VALUETYPE_STRING,
+	OBJDB_VALUETYPE_ANY,
+} objdb_value_types_t;
+
 typedef enum {
 	OBJECT_TRACK_DEPTH_ONE,
 	OBJECT_TRACK_DEPTH_RECURSIVE
@@ -68,18 +81,18 @@ typedef void (*object_key_change_notify_fn_t)(
 	const void *key_value_pt, size_t key_value_len,
 	void *priv_data_pt);
 
-typedef void (*object_create_notify_fn_t) (unsigned int parent_object_handle,
-hdb_handle_t object_handle,
-const void *name_pt, size_t name_len,
-void *priv_data_pt);
+typedef void (*object_create_notify_fn_t) (hdb_handle_t parent_object_handle,
+	hdb_handle_t object_handle,
+	const void *name_pt, size_t name_len,
+	void *priv_data_pt);
 
-typedef void (*object_destroy_notify_fn_t) (unsigned int parent_object_handle,
-					    const void *name_pt,
-					    size_t name_len,
-											void *priv_data_pt);
+typedef void (*object_destroy_notify_fn_t) (hdb_handle_t parent_object_handle,
+	const void *name_pt,
+	size_t name_len,
+	void *priv_data_pt);
 
 typedef void (*object_reload_notify_fn_t) (objdb_reload_notify_type_t, int flush,
-											void *priv_data_pt);
+	void *priv_data_pt);
 
 struct object_valid {
 	char *object_name;
@@ -235,6 +248,27 @@ struct objdb_iface_ver0 {
 		const void *key_name,
 		size_t key_len,
 		unsigned int *value);
+
+	int (*object_key_create_typed) (
+		hdb_handle_t object_handle,
+		const char *key_name,
+		const void *value,
+		size_t value_len,
+		objdb_value_types_t type);
+
+	int (*object_key_get_typed) (
+		hdb_handle_t object_handle,
+		const char *key_name,
+		void **value,
+		size_t *value_len,
+		objdb_value_types_t *type);
+
+	int (*object_key_iter_typed) (
+		hdb_handle_t parent_object_handle,
+		char **key_name,
+		void **value,
+		size_t *value_len,
+		objdb_value_types_t *type);
 };
 
 #endif /* OBJDB_H_DEFINED */
