@@ -6,7 +6,7 @@
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -36,19 +36,24 @@
 
 #include <stdio.h>
 #include <syslog.h>
-#include "../exec/logsys.h"
+
+#include <corosync/engine/logsys.h>
 
 LOGSYS_DECLARE_SYSTEM ("logsystestsubsystems",
-	LOG_MODE_OUTPUT_STDERR | LOG_MODE_OUTPUT_SYSLOG_THREADED, 
+	LOGSYS_MODE_OUTPUT_STDERR | LOGSYS_MODE_OUTPUT_SYSLOG,
+	0,
 	NULL,
+	LOGSYS_LEVEL_INFO,
 	LOG_DAEMON,
-	"[%8s] %b",
-	100000);
+	LOGSYS_LEVEL_INFO,
+	NULL,
+	1000000);
 
 extern void logsys_s1_print (void);
 extern void logsys_s2_print (void);
 
 int main (void) {
+	logsys_fork_completed();
 	logsys_s1_print();
 	logsys_s2_print();
 	return (0);
